;JCM-1 AY-3-8910 Sound Chip Tester
; Tests all channels on all units to ensure they work properly.
; Based on "SOUND CHIP DRIVER PROGRAM" by Eric C. Rehnke
;https://www.atarimagazines.com/computeii/issue3/page4.php

.target "65C02", 64	; We're running a 65C02 Processor

SndChip	 = $D020
OutRegB	 = SndChip
DDRB	 = SndChip+2
DDRA	 = SndChip+3
OutRegA	 = SndChip+15

m_cmdprompt	 = $e022
m_printimm	 = $e19e
acia0_sendbyte = $e1e3

AYSelect	= $80	;Holds which chip we're talking to.

*=$0300

;************************************
;Test Tones	
TestChips
	LDA #0
	STA AYSelect	;Save the unit choice to Zero Page
	
MakeSound
	JSR ClrRegs
	
	;Basic parameters for tones.
	LDA #$38	;00111000	 Basically, disable noise and enable tone on all channels
	LDX #7		;Mixer Control - I/O Enable			(R7 octal)
	JSR SoundOut

	LDA #$07	;xxx1000 - Amplitude "mode"
	LDX #8		;Amplitude Control					(R10 octal)
	JSR SoundOut
	
	LDA #$0B	;xxx1000 - Amplitude "mode"
	LDX #9		;Amplitude Control					(R11 octal)
	JSR SoundOut
	
	LDA #$0F	;xxx1000 - Amplitude "mode"
	LDX #10		;Amplitude Control					(R12 octal)
	JSR SoundOut
	
	JSR TestMsg
	JSR M_PrintImm
	.byte "A, Tone",$0D,$0A,$00
	
		LDA #$77	;Tone Period A, fine tune
		LDX #0		;Tone Generator Control (R0 octal)
		JSR SoundOut
		LDA #$0		;Tone Period A, Coarse tune
		LDX #1		;Tone Generator Control (R1 octal)
		JSR SoundOut
		JSR WaitLoop
		LDA #$00	;Silence the channel
		LDX #8		;Amplitude Control		(R10 octal)
		JSR SoundOut


	JSR TestMsg
	JSR M_PrintImm
	.byte "B, Tone",$0D,$0A,$00			
		LDA #$6A	;Tone Period B, fine tune
		LDX #2		;Tone Generator Control (R2 octal)
		JSR SoundOut
		LDA #0		;Tone Period B, Coarse tune
		LDX #3		;Tone Generator Control (R3 octal)
		JSR SoundOut
		JSR WaitLoop
		LDA #$00	;Silence the channel
		LDX #9		;Amplitude Control		(R11 octal)
		JSR SoundOut
	
	JSR TestMsg
	JSR M_PrintImm
	.byte "C, Tone",$0D,$0A,$00			
		LDA #$5F	;Tone Period C, fine tune
		LDX #4		;Tone Generator Control (R4 octal)
		JSR SoundOut
		LDA #$0		;Tone Period C, Coarse tune
		LDX #5		;Tone Generator Control (R5 octal)
		JSR SoundOut
		JSR WaitLoop
		LDA #$00	;Silence the channel
		LDX #10		;Amplitude Control		(R12 octal)
		JSR SoundOut
		
;************************************
;Test Noise	
	JSR ClrRegs
	
	;Basic noise channel parameters.
	LDA #$0		;Noise Frequency
	LDX #6		;Noise Generator Control (R6 octal)
	JSR SoundOut
	
	LDA #$0F	;xxx1000 - Amplitude "mode"
	LDX #8		;Amplitude Control					(R10 octal)
	JSR SoundOut
	
	LDA #$0B	;xxx1000 - Amplitude "mode"
	LDX #9		;Amplitude Control					(R11 octal)
	JSR SoundOut
	
	LDA #$10	;xxx1000 - Amplitude "mode"
	LDX #10		;Amplitude Control					(R12 octal)
	JSR SoundOut
	
	LDA #0		;00000000
	LDX #13		;Envelope Generator, Fine Tune		(R15 octal)
	JSR SoundOut
	
	LDA #$38	;00111000
	LDX #12		;Envelope Generator, Coarse Tune	(R14 octal)
	JSR SoundOut
	
	JSR TestMsg
	JSR M_PrintImm
	.byte "A, Noise",$0D,$0A,$00
		LDA #$37	;00 110 111	 Noise Channel A
		LDX #7		;Mixer Control - I/O Enable	(R7 octal)
		JSR SoundOut
		JSR WaitLoop
		LDA #$00	;xxx1000 - Amplitude "mode"
		LDX #8		;Amplitude Control			(R10 octal)
		JSR SoundOut
		
		
	JSR TestMsg
	JSR M_PrintImm
	.byte "B, Noise",$0D,$0A,$00			
		LDA #$2F	;00 101 111	 Noise Channel B
		LDX #7		;Mixer Control - I/O Enable	(R7 octal)
		JSR SoundOut
		JSR WaitLoop
		LDA #$00	;xxx1000 - Amplitude "mode"
		LDX #9		;Amplitude Control			(R11 octal)
		JSR SoundOut
		
	JSR TestMsg
	JSR M_PrintImm
	.byte "C, Noise",$0D,$0A,$00			
		LDA #$1F	;00 011 111	 Noise Channel C
		LDX #7		;Mixer Control - I/O Enable	(R7 octal)
		JSR SoundOut
		JSR WaitLoop
		LDA #$00	;xxx1000 - Amplitude "mode"
		LDX #10		;Amplitude Control			(R12 octal)
		JSR SoundOut

LDA AYSelect
BEQ OnceMore		;Not Done yet.
JMP m_cmdprompt		;Done! Return to prompt.

OnceMore
	LDA #1			;Select Unit One
	STA AYSelect	;Save the unit choice to Zero Page
	JMP MakeSound


	
;** SUBROUTINES **


;************************************
;Send sound to the sound chip
;A should be the value to send.
;X should be the chip register number to send to
SoundOut	TAY           ;Backup A, since Latch is destructive.  
			JSR Latch
			JSR Write
			RTS


;************************************
;Performs the Latch cycle to select a specific register.			
;Required because the AY-3-8910 uses the same eight pins for both address and data.

Latch		LDA #$FF	;Set all pins to outputs.
			STA DDRA
			STA DDRB
			STX OutRegA
			
			LDA AYSelect
			BNE Latch1
			LDA #3		;Select Unit 0
			BNE LatchIt
Latch1		LDA #12		;Select Unit 1			
LatchIt		STA OutRegB
			LDA #0
			STA OutRegB
			RTS


;************************************
;Performs the write cycle after a successful latch has been completed.
Write		TYA   
			STA OutRegA
			LDA AYSelect
			BNE Write1
			LDA #2		;Select Unit 0
			BNE WriteIt
	Write1 	LDA #8		;Select Unit 1
	WriteIt	STA OutRegB
			LDA #0
			STA OutRegB
			RTS

;************************************
;Zeroes all registers in the sound chip.			
ClrRegs		LDX #0
ClrLoop		LDA #0
			JSR SoundOut
			INX
			CPX #17
			BNE ClrLoop
			RTS

;************************************
; Timing wait loop.
WaitLoop
		LDX #0
		LDY #0
WaitA 	INX
		BNE WaitA
		INY
		BNE WaitA
		RTS			

;************************************
; Print Testing Message
TestMsg
	JSR M_PrintImm
	.byte "Testing Unit ",$00
	CLC
	LDA AYSelect
	ADC #48
	JSR acia0_sendbyte
	JSR M_PrintImm
	.byte ", Channel ",$00
RTS

